function OBSA(M,N,StartH,StartHT,DerM,DerN,GDerM,GDerN,symflag,s1,s2,inivar)

%
%  OBSA(M,N,StartH,StartHT,DerM,DerN,GDerM,GDerN,symflag,s1,s2,inivar)  
%
%    For general omnidirectionally balanced multiwavelets design.
%
%    M: length of basis lowpass filter H
%    N: length of dual lowpass filter H~, 
%       M and N must be either both even or both odd
%    StartH: the start index of H's nonzero coefficient
%    StartHT: the start index of H~'s nonzero coefficient
%    DerM: the vanishing moments required for H
%    DerN: the vanishing moments required for H~
%    GDerM: the vanishing moments required for G
%    GDerN: the vanishing moments required for G~
%    symflag: flag indicates the desired symmetry 
%             -1 - symmetric-antisymmetric
%              1 - symmetric
%    s1: the index of which result to select in solutions of H, H~
%    s2: the index of which to select in solutions of G, G~
%    inivar: the initial value of gradient descending search method
%

%
% NOTE:
% RELATION BETWEEN FILTER SUPPORT INTERVALS AND VARIABLES USED HERE:
%   StartH = K_l;
%   StartHT = \tilde{K}_l;
%   M = K_u - K_l + 1;
%   N = \tilde{K}_u - \tilde{K}_l + 1;
%   StartG = L_l;
%   StartGT = \tilde{L}_l;
%   GM = L_u - L_l + 1;
%   GN = \tilde{L}_u - \tilde{L}_l + 1;
%

if mod(M-N,2) ~= 0
   disp('The length of basis and dual filters must be both even or both odd.');
end

Sr = [1 0; 0 symflag]
I = eye(2);
O = zeros(2);
J = [0 1; 1 0];

%
% SYMOBLIC SET UP OF MATRIX COEFFICIENTS, SA CONDITION
%

for l = 1:ceil((M-1)/2)
   index = 4*(l-1);
   eval(['syms x' num2str(index+1) ...
         ' x' num2str(index+2) ...
         ' x' num2str(index+3) ...
         ' x' num2str(index+4)]);
   eval(['P' num2str(l-1) ...
         ' = [x' num2str(index+1) ...
         ' x' num2str(index+2) ...
         '; x' num2str(index+3) ...
         ' x' num2str(index+4) ...
              ']']);
   eval(['P' num2str(M-l) ...
         ' = Sr*P' num2str(l-1) ...
         '*Sr']);
end
indexP = index+4;
if mod(M,2) ~= 0
   index = 2*(M-1);
   eval(['syms x' num2str(index+1) ...
         ' x' num2str(index+2)]);
   eval(['P' num2str((M-1)/2) ...
         ' = [x' num2str(index+1) ' 0;' ...
         ' 0 x' num2str(index+2) ']']);
   indexP = index+2;
end

for l = 1:ceil((N-1)/2)
   index = 4*(l-1);
   eval(['syms y' num2str(index+1) ...
         ' y' num2str(index+2) ...
         ' y' num2str(index+3) ...
         ' y' num2str(index+4)]);
   eval(['Q' num2str(l-1) ...
         ' = [y' num2str(index+1) ...
         ' y' num2str(index+2) ...
         '; y' num2str(index+3) ...
         ' y' num2str(index+4) ...
              ']']);
   eval(['Q' num2str(N-l) ...
         ' = Sr*Q' num2str(l-1) ...
         '*Sr']);
end
indexQ = index+4;
if mod(N,2) ~= 0
   index = 2*(N-1);
   eval(['syms y' num2str(index+1) ...
         ' y' num2str(index+2)]);
   eval(['Q' num2str((N-1)/2) ...
         ' = [y' num2str(index+1) ' 0;' ...
         ' 0 y' num2str(index+2) ']']);
   indexQ = index+2;
end


% 
% DESIGN CONSTRAINTS, PR, OB, VANISHING MOMENTS
%

syms z;

P = 0;
for l = 0:M-1
   eval(['P = P+P' num2str(l) '*(z^' num2str(l+StartH) ');']);
end
P = simple(P/2)

Q = 0;
for l = 0:N-1
  eval(['Q = Q + Q' num2str(l) '*(z^' num2str(l+StartHT) ');']);
end
Q = simple(Q/2)

Dis = max(StartH+M-1, StartHT+N-1) - min(StartH, StartHT) + 1;
Num = ceil(Dis/2);

for l = 1-Num:Num-1

  if l == 0
     eval(['E1' num2str(l+Num) '= -2*I;']);
  else 
     eval(['E1' num2str(l+Num) ' = 0;']);
  end

  for k = 0:M-1
      note = StartH+k+2*l-StartHT;
      if note < N & note >= 0
          str = ['Q' num2str(note)];
      else
          str = 'O';
      end 
      eval(['E1' num2str(l+Num) ' = E1' num2str(l+Num) ...
            '+ P' num2str(k) '*' str '.'';']);
  end

      eval(['E1' num2str(l+Num) ' = simple(E1' num2str(l+Num) ')']);
      
      if l == 1-Num
         eval(['eqns = matrixtoeqn(E11);']);
      else 
         eval(['eqns = [eqns '','' matrixtoeqn(E1' num2str(l+Num) ')];']);
      end
end

E2 =  subs(P,'z',1) - I;
E2 = simple(E2)
eqns = [eqns ',' matrixtoeqn(E2)];

for l = 0:DerM
   eval(['E3' num2str(l) '= subs(diff(P,''z'',' num2str(l) ...
         '),''z'',-1);']);
   eval(['E3' num2str(l) '= simple(E3' num2str(l) ')']);
   eval(['eqns = [eqns '','' matrixtoeqn(E3' num2str(l) ')];']);
end

E4 = subs(Q,'z',1) - I;
E4 = simple(E4)
eqns = [eqns ',' matrixtoeqn(E4)];

for l = 0:DerN
   eval(['E5' num2str(l) '= subs(diff(Q,''z'',' num2str(l) ...
         '),''z'',-1);']);
   eval(['E5' num2str(l) '= simple(E5' num2str(l) ')']);
   eval(['eqns = [eqns '','' matrixtoeqn(E5' num2str(l) ')];']);
end


%
% SOLVING EQUATIONS
%

strg = 'cel = {';

for l = 1:indexP
   strg = [strg ' ''x' num2str(l) ''''];
end
for l = 1:indexQ
   strg = [strg ' ''y' num2str(l) ''''];
end
strg = [strg '};'];
eval(strg);
cel = sort(cel);

str = [];
for l = 1:length(cel)
   if l ~= 1
      str = [str ','];
   end
   str = [str cel{l}];
end

eval(['[' str '] = solve(eqns,' str ')']);


%
% SELECT SOLUTION
%

for l = 1:indexP
    eval(['X' num2str(l) '=x' num2str(l) '(s1);']);
end
for l = 1:indexQ
    eval(['Y' num2str(l) '=y' num2str(l) '(s1);']);
end

for l = 1:ceil((M-1)/2)
   index = 4*(l-1);
   eval(['P' num2str(l-1) ...
         ' = [X' num2str(index+1) ...
         ' X' num2str(index+2) ...
         '; X' num2str(index+3) ...
         ' X' num2str(index+4) ...
              ']']);
   eval(['P' num2str(M-l) ...
         ' = Sr*P' num2str(l-1) ...
         '*Sr']);
end
if mod(M,2) ~= 0
   index = 2*(M-1);
   eval(['P' num2str((M-1)/2) ...
         ' = [X' num2str(index+1) ' 0;' ...
         ' 0 X' num2str(index+2) ']']);
end

for l = 1:ceil((N-1)/2)
   index = 4*(l-1);
   eval(['Q' num2str(l-1) ...
         ' = [Y' num2str(index+1) ...
         ' Y' num2str(index+2) ...
         '; Y' num2str(index+3) ...
         ' Y' num2str(index+4) ...
              ']']);
   eval(['Q' num2str(N-l) ...
         ' = Sr*Q' num2str(l-1) ...
         '*Sr']);
end
if mod(N,2) ~= 0
   index = 2*(N-1);
   eval(['Q' num2str((N-1)/2) ...
         ' = [Y' num2str(index+1) ' 0;' ...
         ' 0 Y' num2str(index+2) ']']);
end

P = 0;
for l = 0:M-1
   eval(['P = P+P' num2str(l) '*(z^' num2str(l+StartH) ');']);
end
P = simple(P/2)
Q = 0;
for l = 0:N-1
  eval(['Q = Q + Q' num2str(l) '*(z^' num2str(l+StartHT) ');']);
end
Q = simple(Q/2)

%
% FILTER OPTIMIZATION
%

[cel, res] = FilterOptimize(P, Q, 0, inivar)

for l = 1:length(cel)
     eval([cel{l} '= res(l)'])
end

H = [];
for l = 0:M-1
   eval(['P' num2str(l) ' = simple(subs(P' num2str(l) '));'], ...
        ['P' num2str(l) ' = double(subs(P' num2str(l) '));']);
   eval(['H = [H; P' num2str(l) '];']);
end
HT = [];
for l = 0:N-1
   eval(['Q' num2str(l) ' = simple(subs(Q' num2str(l) '));'], ...
        ['Q' num2str(l) ' = double(subs(Q' num2str(l) '));']);
   eval(['HT = [HT; Q' num2str(l) '];']);
end

if (~isreal(H) | ~isreal(HT)) 
     H
     HT
     error('Complex result obtained');
     return;
end

double(H)
double(HT)

clear eqns;

% 
% DESIGNING HIGHPASS FILTERS G & G~ 
%

StartG = -(StartHT+N-1-1);
GM = N;
StartGT = -(StartH+M-1-1);
GN = M;

for l = 1:ceil((GM-1)/2)
   index = 4*(l-1);
   eval(['syms r' num2str(index+1) ...
         ' r' num2str(index+2) ...
         ' r' num2str(index+3) ...
         ' r' num2str(index+4)]);
   eval(['GP' num2str(l-1) ...
         ' = [r' num2str(index+1) ...
         ' r' num2str(index+2) ...
         '; r' num2str(index+3) ...
         ' r' num2str(index+4) ...
              ']']);
   eval(['GP' num2str(GM-l) ...
         ' = Sr*GP' num2str(l-1) ...
         '*Sr']);
end
indexP = index+4;
if mod(GM,2) ~= 0
   index = 2*(GM-1);
   eval(['syms r' num2str(index+1) ...
         ' r' num2str(index+2)]);
   eval(['GP' num2str((GM-1)/2) ...
         ' = [r' num2str(index+1) ' 0;' ...
         ' 0 r' num2str(index+2) ']']);
   indexP = index+2;
end

for l = 1:ceil((GN-1)/2)
   index = 4*(l-1);
   eval(['syms t' num2str(index+1) ...
         ' t' num2str(index+2) ...
         ' t' num2str(index+3) ...
         ' t' num2str(index+4)]);
   eval(['GQ' num2str(l-1) ...
         ' = [t' num2str(index+1) ...
         ' t' num2str(index+2) ...
         '; t' num2str(index+3) ...
         ' t' num2str(index+4) ...
              ']']);
   eval(['GQ' num2str(GN-l) ...
         ' = Sr*GQ' num2str(l-1) ...
         '*Sr']);
end
indexQ = index+4;
if mod(GN,2) ~= 0
   index = 2*(GN-1);
   eval(['syms t' num2str(index+1) ...
         ' t' num2str(index+2)]);
   eval(['GQ' num2str((GN-1)/2) ...
         ' = [t' num2str(index+1) ' 0;' ...
         ' 0 t' num2str(index+2) ']']);
   indexQ = index+2;
end


%
% DESIGN CONSTRAINTS, ORTHOGONALITY, VANISHING MOMENTS 
%

GP = 0;
for l = 0:GM-1
   eval(['GP = GP+GP' num2str(l) '*(z^' num2str(l+StartG) ');']);
end
GP = simple(GP/2)

GQ = 0;
for l = 0:GN-1
  eval(['GQ = GQ + GQ' num2str(l) '*(z^' num2str(l+StartGT) ');']);
end
GQ = simple(GQ/2)

Dis = max(StartH+M-1, StartGT+GN-1) - min(StartH, StartGT) + 1;
Num = ceil(Dis/2);
eqns = [];

for l = 1-Num:Num-1

  eval(['E1' num2str(l+Num) ' = 0;']);

  for k = 0:M-1
      note = StartH+k+2*l-StartGT;
      if note < GN & note >= 0
          str = ['GQ' num2str(note)];
      else
          str = 'O';
      end
      eval(['E1' num2str(l+Num) ' = E1' num2str(l+Num) ...
                 '+ P' num2str(k) '*' str '.'';']);
  end
      
      flag = eval(['isequal(E1' num2str(l+Num) ', zeros(2,2))']);

      if ~flag
        eval(['E1' num2str(l+Num) ' = simple(E1' num2str(l+Num) ')'], ...
             ['E1' num2str(l+Num) ]);
      
        if isempty(eqns)
            eval(['eqns = matrixtoeqn(E1' num2str(l+Num) ');']);
         else 
            eval(['eqns = [eqns '','' matrixtoeqn(E1' num2str(l+Num) ')];']);
        end
      end
end


Dis = max(StartG+GM-1, StartGT+GN-1) - min(StartG, StartGT) + 1;
Num = ceil(Dis/2);

for l = 1-Num:Num-1

  if l == 0
     eval(['E2' num2str(l+Num) ' = -2*I;']);
  else 
     eval(['E2' num2str(l+Num) ' = 0;']);
  end

  for k = 0:GM-1
      note = StartG+k+2*l-StartGT;
      if note < GN & note >= 0
          str = ['GQ' num2str(note)];
      else
          str = 'O';
      end
      eval(['E2' num2str(l+Num) ' = E2' num2str(l+Num) ...
                 '+ GP' num2str(k) '*' str '.'';']);
  end

      eval(['E2' num2str(l+Num) ' = simple(E2' num2str(l+Num) ')'], ...
           ['E2' num2str(l+Num) ]);
      
      eval(['eqns = [eqns '','' matrixtoeqn(E2' num2str(l+Num) ')];']);
end

for l = 0:GDerM
   eval(['E4' num2str(l) '= subs(diff(GP,''z'',' num2str(l) ...
         '),''z'',-1);']);
   eval(['E4' num2str(l) '= simple(E4' num2str(l) ')']);
   eval(['eqns = [eqns '','' matrixtoeqn(E4' num2str(l) ')];']);
end

for l = 0:GDerN
   eval(['E5' num2str(l) '= subs(diff(GQ,''z'',' num2str(l) ...
         '),''z'',-1);']);
   eval(['E5' num2str(l) '= simple(E5' num2str(l) ')']);
   eval(['eqns = [eqns '','' matrixtoeqn(E5' num2str(l) ')];']);
end

%
% SOLVING EQUATIONS
%

strg = 'cel = {';

for l = 1:indexP
   strg = [strg ' ''r' num2str(l) ''''];
end
for l = 1:indexQ
   strg = [strg ' ''t' num2str(l) ''''];
end
strg = [strg '};'];
eval(strg);
cel = sort(cel);

str = [];
for l = 1:length(cel)
   if l ~= 1
      str = [str ','];
   end
   str = [str cel{l}];
end

eval(['[' str '] = solve(eqns,' str ')']);


%
% SELECT SOLUTION
%

for l = 1:indexP
    eval(['R' num2str(l) '= r' num2str(l) '(s2);']);
end
for l = 1:indexQ
    eval(['T' num2str(l) '= t' num2str(l) '(s2);']);
end

for l = 1:ceil((GM-1)/2)
   index = 4*(l-1);
   eval(['GP' num2str(l-1) ...
         ' = [R' num2str(index+1) ...
         ' R' num2str(index+2) ...
         '; R' num2str(index+3) ...
         ' R' num2str(index+4) ...
              ']']);
   eval(['GP' num2str(GM-l) ...
         ' = Sr*GP' num2str(l-1) ...
         '*Sr']);
end
if mod(GM,2) ~= 0
   index = 2*(GM-1);
   eval(['GP' num2str((GM-1)/2) ...
         ' = [R' num2str(index+1) ' 0;' ...
         ' 0 R' num2str(index+2) ']']);
end

for l = 1:ceil((GN-1)/2)
   index = 4*(l-1);
   eval(['GQ' num2str(l-1) ...
         ' = [T' num2str(index+1) ...
         ' T' num2str(index+2) ...
         '; T' num2str(index+3) ...
         ' T' num2str(index+4) ...
              ']']);
   eval(['GQ' num2str(GN-l) ...
         ' = Sr*GQ' num2str(l-1) ...
         '*Sr']);
end
if mod(GN,2) ~= 0
   index = 2*(GN-1);
   eval(['GQ' num2str((GN-1)/2) ...
         ' = [T' num2str(index+1) ' 0;' ...
         ' 0 T' num2str(index+2) ']']);
end

GP = 0;
for l = 0:GM-1
   eval(['GP = GP+GP' num2str(l) '*(z^' num2str(l+StartG) ');']);
end
GP = simple(GP/2)

GQ = 0;
for l = 0:GN-1
  eval(['GQ = GQ + GQ' num2str(l) '*(z^' num2str(l+StartGT) ');']);
end
GQ = simple(GQ/2)

%
% FILTER OPTIMIZATION 
%

[cel, res] = FilterOptimize(GP, GQ, 1, inivar)

for l = 1:length(cel)
     eval([cel{l} '= res(l)'])
end

G = [];
for l = 0:GM-1
   eval(['GP' num2str(l) ' = simple(subs(GP' num2str(l) '));'], ...
        ['GP' num2str(l) ' = double(subs(GP' num2str(l) '));']);
   eval(['G = [G; GP' num2str(l) '];']);
end
GT = [];
for l = 0:GN-1
   eval(['GQ' num2str(l) ' = simple(subs(GQ' num2str(l) '));'], ...
        ['GQ' num2str(l) ' = double(subs(GQ' num2str(l) '));']);
   eval(['GT = [GT; GQ' num2str(l) '];']);
end

if (~isreal(G) | ~isreal(GT)) 
     G 
     GT
     error('Complex result obtained');
     return;
end


%
%  SAVE IN FILES - QccPack vfb format
%
format long

st = min([StartG, StartGT, StartHT, StartH]);

ID='VFB0.17';
Head = 1;
H = [zeros((StartH-st)*2,2); H]
H = double(H./sqrt(2));
H1 = [2;size(H,1)/2;0];

G = [zeros((StartG-st)*2,2);  G]
G = double(G./sqrt(2));
G1 = [2;size(G,1)/2;0];

HT = [zeros((StartHT-st)*2,2); HT]
HT = double(HT./sqrt(2));
H2 = [2;size(HT,1)/2;0];

GT = [zeros((StartGT-st)*2,2); GT]
GT = double(GT./sqrt(2));
G2 = [2;size(GT,1)/2;0];

format
fid=fopen('log.vfb','w');
fprintf(fid,'%s\n\n',ID);
fprintf(fid,'%d\n\n',Head);
fprintf(fid,'%d\n',H1); 
fprintf(fid,' %.16e %.16e\n %.16e %.16e\n',H');
fprintf(fid,'\n\n');
fprintf(fid,'%d\n',G1); 
fprintf(fid,' %.16e %.16e\n %.16e %.16e\n',G');
fprintf(fid,'\n\n');
fprintf(fid,'%d\n',H2);
fprintf(fid,' %.16e %.16e\n %.16e %.16e\n',HT');
fprintf(fid,'\n\n');
fprintf(fid,'%d\n',G2);
fprintf(fid,' %.16e %.16e\n %.16e %.16e\n',GT');
fclose(fid);

save 'log.mat';
return;

